@@ -51,7 +51,6 @@ from Gnumed.business import gmExportArea
 from Gnumed.business import gmBilling
 from Gnumed.business import gmAutoHints
 from Gnumed.business.gmDocuments import cDocumentFolder
-from Gnumed.business.gmChartPulling import tui_chart_puller
 
 
 _log = logging.getLogger('gm.person')
@@ -966,23 +965,14 @@ class cPerson(gmBusinessDBObject.cBusinessDBObject):
 		})
 
 		# transfer names
-		# 0) force pat2del to be a unique identity from pat2keep
-		#    should identical names happen to occur
-		queries.append ({
-			'cmd': u"""
-				UPDATE dem.identity SET
-					comment = coalesce(comment || ' ', '') || '(%s)'
-				WHERE
-					pk = %%(pat2del)s
-			""" % distinguisher,
-			'args': args
-		})
-
 		# 1) disambiguate names in old patient
 		queries.append ({
 			'cmd': u"""
 				UPDATE dem.names d_n1 SET
-					lastnames = lastnames || ' (%s)'
+					lastnames = lastnames || coalesce (
+						' (from "' || (SELECT comment FROM dem.identity WHERE pk = %%(pat2del)s) || '")',
+						' (%s)'
+					)
 				WHERE
 					d_n1.id_identity = %%(pat2del)s
 						AND
@@ -1001,29 +991,34 @@ class cPerson(gmBusinessDBObject.cBusinessDBObject):
 		queries.append ({
 			'cmd': u"""
 				UPDATE dem.names SET
-					id_identity = %(pat2keep)s
-				WHERE id_identity = %(pat2del)s AND active IS false""",
+					id_identity = %%(pat2keep)s,
+					comment = coalesce(comment || ' (%s)', '%s')
+				WHERE
+					id_identity = %%(pat2del)s
+						AND
+					active IS false
+				""" % (distinguisher, distinguisher),
 			'args': args
 		})
-		# 3) copy active name (because each identity MUST have at least one
+		# 3) copy active name (because each identity MUST have at most one
 		#    *active* name so we can't just UPDATE over to pat2keep),
-		#    also, needs de-duplication or else it would conflict with *itself*
-		#    on pat2keep, said de-duplication happened in step 0 above
-		#    whereby pat2del is made unique by means of adding a pseudo-random
-		#    dem.identity.comment
+#		#    also, needs de-duplication or else it would conflict with *itself*
+#		#    on pat2keep, said de-duplication happened in step 0 above
+#		#    whereby pat2del is made unique by means of adding a pseudo-random
+#		#    dem.identity.comment
 		queries.append ({
 			'cmd': u"""
 				INSERT INTO dem.names (
 					id_identity, active, lastnames, firstnames, preferred, comment
 				)
 					SELECT
-						%%(pat2keep)s, false, lastnames, firstnames, preferred, comment || ' (%s)'
+						%%(pat2keep)s, false, lastnames, firstnames, preferred, coalesce(comment || ' (%s)', '%s')
 					FROM dem.names d_n
 					WHERE
 						d_n.id_identity = %%(pat2del)s
 							AND
 						d_n.active IS true
-				""" % distinguisher,
+				""" % (distinguisher, distinguisher),
 			'args': args
 		})
 
@@ -1730,7 +1725,7 @@ class cPerson(gmBusinessDBObject.cBusinessDBObject):
 		# -> DOB is today
 		return False
 
-	current_birthday_passed = property(_get_current_birthday_passed, lambda x:x)
+	current_birthday_passed = property(_get_current_birthday_passed)
 
 	#----------------------------------------------------------------------
 	def _get_birthday_this_year(self):
@@ -1743,7 +1738,33 @@ class cPerson(gmBusinessDBObject.cBusinessDBObject):
 			strict = False
 		)
 
-	birthday_this_year = property(_get_birthday_this_year, lambda x:x)
+	birthday_this_year = property(_get_birthday_this_year)
+
+	#----------------------------------------------------------------------
+	def _get_birthday_next_year(self):
+		if self['dob'] is None:
+			return None
+		now = gmDateTime.pydt_now_here()
+		return gmDateTime.pydt_replace (
+			dt = self['dob'],
+			year = now.year + 1,
+			strict = False
+		)
+
+	birthday_next_year = property(_get_birthday_next_year)
+
+	#----------------------------------------------------------------------
+	def _get_birthday_last_year(self):
+		if self['dob'] is None:
+			return None
+		now = gmDateTime.pydt_now_here()
+		return gmDateTime.pydt_replace (
+			dt = self['dob'],
+			year = now.year - 1,
+			strict = False
+		)
+
+	birthday_last_year = property(_get_birthday_last_year, lambda x:x)
 
 	#----------------------------------------------------------------------
 	# practice related
@@ -1812,30 +1833,34 @@ class cPerson(gmBusinessDBObject.cBusinessDBObject):
 	#----------------------------------------------------------------------
 	def get_dirname(self):
 		"""Format patient demographics into patient specific path name fragment."""
-		#return (u'%s-%s%s-%s' % (
-		return (u'%s-%s-%s' % (
-			self._payload[self._idx['lastnames']].replace(u' ', u'_'),
-			self._payload[self._idx['firstnames']].replace(u' ', u'_'),
-			# privacy
-			#gmTools.coalesce(self._payload[self._idx['preferred']], u'', template_initial = u'-(%s)').replace(u' ', u'_'),
+
+		return gmTools.fname_sanitize(u'%s-%s-%s' % (
+			self._payload[self._idx['lastnames']],
+			self._payload[self._idx['firstnames']],
 			self.get_formatted_dob(format = '%Y-%m-%d', encoding = gmI18N.get_encoding())
-		)).replace (
-			u"'", u""
-		).replace (
-			u'"', u''
-		).replace (
-			u'/', u'_'
-		).replace (
-			u'\\', u'_'
-		).replace (
-			u'~', u''
-		).replace (
-			u'|', u'_'
-		).replace (
-			u'*', u''
-		).replace (
-			u'\u2248', u''			# "approximately", having been added by dob_is_estimated
-		)
+		))
+#		return (u'%s-%s-%s' % (
+#			self._payload[self._idx['lastnames']].replace(u' ', u'_'),
+#			self._payload[self._idx['firstnames']].replace(u' ', u'_'),
+#			self.get_formatted_dob(format = '%Y-%m-%d', encoding = gmI18N.get_encoding())
+#		)).replace (
+#			u"'", u""
+#		).replace (
+#			u'"', u''
+#		).replace (
+#			u'/', u'_'
+#		).replace (
+#			u'\\', u'_'
+#		).replace (
+#			u'~', u''
+#		).replace (
+#			u'|', u'_'
+#		).replace (
+#			u'*', u''
+#		).replace (
+#			u'\u2248', u''			# "approximately", having been added by dob_is_estimated
+#		)
+
 
 	dirname = property(get_dirname, lambda x:x)
 
@@ -1863,29 +1888,6 @@ def turn_identity_into_patient(pk_identity):
 #============================================================
 # helper functions
 #------------------------------------------------------------
-#_spin_on_emr_access = None
-#
-#def set_emr_access_spinner(func=None):
-#	if not callable(func):
-#		_log.error('[%] not callable, not setting _spin_on_emr_access', func)
-#		return False
-#
-#	_log.debug('setting _spin_on_emr_access to [%s]', func)
-#
-#	global _spin_on_emr_access
-#	_spin_on_emr_access = func
-#
-
-#------------------------------------------------------------
-_pull_chart = tui_chart_puller
-
-def set_chart_puller(chart_puller):
-	if not callable(chart_puller):
-		raise TypeError('chart puller <%s> not callable' % chart_puller)
-	global _pull_chart
-	_pull_chart = chart_puller
-	_log.debug('setting chart puller to <%s>', chart_puller)
-
 _yield = lambda x:x
 
 def set_yielder(yielder):
@@ -1907,6 +1909,7 @@ class cPatient(cPerson):
 		self.__emr_access_lock = threading.Lock()
 		self.__emr = None
 		self.__doc_folder = None
+
 	#--------------------------------------------------------
 	def cleanup(self):
 		"""Do cleanups before dying.
@@ -1918,80 +1921,62 @@ class cPatient(cPerson):
 		if self.__doc_folder is not None:
 			self.__doc_folder.cleanup()
 		cPerson.cleanup(self)
+
 	#----------------------------------------------------------------
 	def ensure_has_allergy_state(self, pk_encounter=None):
 		from Gnumed.business.gmAllergy import ensure_has_allergy_state
 		ensure_has_allergy_state(encounter = pk_encounter)
 		return True
-	#----------------------------------------------------------
-	#----------------------------------------------------------
-#	def get_emr(self, allow_user_interaction=True):
-#		if not self.__emr_access_lock.acquire(False):
-#			# maybe something slow is happening on the machine
-#			_log.debug('failed to acquire EMR access lock, sleeping for 500ms')
-#			time.sleep(0.5)
-#			if not self.__emr_access_lock.acquire(False):
-#				_log.error('still failed to acquire EMR access lock, aborting')
-#				raise AttributeError('cannot lock access to EMR')
-#
-#		if self.__emr is None:
-#			self.__emr = gmClinicalRecord.cClinicalRecord(aPKey = self._payload[self._idx['pk_identity']], allow_user_interaction = allow_user_interaction)
-#			self.__emr.gender = self['gender']
-#			self.__emr.dob = self['dob']
-#
-#		self.__emr_access_lock.release()
-#		return self.__emr
 
-	def get_emr(self, allow_user_interaction=True):
+	#----------------------------------------------------------
+	def get_emr(self):
 		_log.debug('accessing EMR for identity [%s], thread [%s]', self._payload[self._idx['pk_identity']], thread.get_ident())
+
+		# fast path: already set, just return it
+		if self.__emr is not None:
+			return self.__emr
+
 		stack_logged = False
-		if not self.__emr_access_lock.acquire(False):
-			got_lock = False
+		got_lock = self.__emr_access_lock.acquire(False)
+		if not got_lock:
 			# do some logging as we failed to get the lock
 			call_stack = inspect.stack()
+			call_stack.reverse()
 			for idx in range(1, len(call_stack)):
 				caller = call_stack[idx]
 				_log.debug('%s[%s] @ [%s] in [%s]', u' '* idx, caller[3], caller[2], caller[1])
 			del call_stack
 			stack_logged = True
 			# now loop a bit
-			for idx in range(100):
+			for idx in range(500):
 				_yield()
 				time.sleep(0.1)
 				_yield()
-				if self.__emr_access_lock.acquire(False):
-					got_lock = True
+				got_lock = self.__emr_access_lock.acquire(False)
+				if got_lock:
 					break
 			if not got_lock:
 				_log.error('still failed to acquire EMR access lock, aborting (thread [%s])', thread.get_ident())
+				self.__emr_access_lock.release()
 				raise AttributeError('cannot lock access to EMR for identity [%s]' % self._payload[self._idx['pk_identity']])
 
-#			# maybe something slow is happening on the machine
-#			_log.debug('failed to acquire EMR access lock, sleeping for 500ms (thread %s)', thread.get_ident())
-#			time.sleep(0.5)
-#			if not self.__emr_access_lock.acquire(False):
-#				_log.error('still failed to acquire EMR access lock, aborting (thread %s)', thread.get_ident())
-#				raise AttributeError('cannot lock access to EMR')
+		_log.debug('pulling chart for identity [%s], thread [%s]', self._payload[self._idx['pk_identity']], thread.get_ident())
+		if not stack_logged:
+			# do some logging as we are pulling the chart for the first time
+			call_stack = inspect.stack()
+			call_stack.reverse()
+			for idx in range(1, len(call_stack)):
+				caller = call_stack[idx]
+				_log.debug('%s[%s] @ [%s] in [%s]', u' '* idx, caller[3], caller[2], caller[1])
+			del call_stack
+			stack_logged = True
 
-		if self.__emr is None:
-			_log.debug('pulling chart for identity [%s], thread [%s]', self._payload[self._idx['pk_identity']], thread.get_ident())
-			if not stack_logged:
-				# do some logging as we are pulling the chart for the first time
-				call_stack = inspect.stack()
-				for idx in range(1, len(call_stack)):
-					caller = call_stack[idx]
-					_log.debug('%s[%s] @ [%s] in [%s]', u' '* idx, caller[3], caller[2], caller[1])
-				del call_stack
-				stack_logged = True
-			#emr = _pull_chart(self._payload[self._idx['pk_identity']])
-			emr = _pull_chart(self)
-			if emr is None:		# user aborted pulling chart
-				_log.info('user aborted pulling chart, returning None')
-				self.__emr_access_lock.release()
-				return None
-			self.__emr = emr
+		self.is_patient = True
+		from Gnumed.business import gmClinicalRecord
+		emr = gmClinicalRecord.cClinicalRecord(aPKey = self._payload[self._idx['pk_identity']])
 
 		_log.debug('returning EMR for identity [%s], thread [%s]', self._payload[self._idx['pk_identity']], thread.get_ident())
+		self.__emr = emr
 		self.__emr_access_lock.release()
 		return self.__emr
 
@@ -2124,10 +2109,11 @@ class gmCurrentPatient(gmBorg.cBorg):
 		self.patient = patient
 		# for good measure ...
 		# however, actually we want to get rid of that
-		self.patient.get_emr()
+		self.patient.emr
 		self.__send_selection_notification()
 
 		return None
+
 	#--------------------------------------------------------
 	def __register_interests(self):
 		gmDispatcher.connect(signal = u'gm_table_mod', receiver = self._on_database_signal)
@@ -2666,7 +2652,7 @@ if __name__ == '__main__':
 	#--------------------------------------------------------
 	def test_current_patient():
 		pat = gmCurrentPatient()
-		print "pat.get_emr()", pat.get_emr()
+		print "pat.emr", pat.emr
 
 	#--------------------------------------------------------
 	def test_ext_id():
