@@ -215,7 +215,7 @@ def act_on_generated_forms(parent=None, forms=None, jobtype=None, episode_name=N
 		if soap.strip() == u'':
 			return
 		pat = gmPerson.gmCurrentPatient()
-		emr = pat.get_emr()
+		emr = pat.emr
 		epi = emr.add_episode(episode_name = episode_name, is_open = False)
 		emr.add_clin_narrative (
 			soap_cat = None,
@@ -228,7 +228,7 @@ def act_on_generated_forms(parent=None, forms=None, jobtype=None, episode_name=N
 			epi = None				# will ask for episode further down
 		else:
 			pat = gmPerson.gmCurrentPatient()
-			emr = pat.get_emr()
+			emr = pat.emr
 			epi = emr.add_episode(episode_name = episode_name, is_open = False)
 
 		for form in forms:
@@ -775,6 +775,7 @@ class cReceiverSelectionDlg(wxgReceiverSelectionDlg.wxgReceiverSelectionDlg):
 
 	#------------------------------------------------------------
 	def __populate_candidates_list(self):
+
 		list_items = [[_('Patient'), self.__patient[u'description_gender'].strip()]]
 		list_data = [(self.__patient[u'description'].strip(), self.__patient.get_addresses(), u'', None)]
 
