@@ -612,7 +612,7 @@ class cProviderInboxPnl(wxgProviderInboxPnl.wxgProviderInboxPnl, gmRegetMixin.cR
 					m['lastnames'],
 					m['firstnames'],
 					gmTools.coalesce(m['l10n_gender'], u'', u' (%s)'),
-					gmDateTime.pydt_strftime(m['dob_only'], u'%Y %b %d', none_str = u''),
+					gmDateTime.pydt_strftime(m['dob'], u'%Y %b %d', none_str = u''),
 					m['pk_patient']
 				))
 			item.append(m['comment'])
@@ -771,11 +771,7 @@ class cProviderInboxPnl(wxgProviderInboxPnl.wxgProviderInboxPnl, gmRegetMixin.cR
 
 		pk_patient = self.__focussed_msg['pk_patient']
 		if pk_patient is not None:
-			#emr = gmClinicalRecord.cClinicalRecord(aPKey = pk_patient, allow_user_interaction = False)
-			from Gnumed.wxpython import gmChartPullingWidgets
-			emr = gmChartPullingWidgets.pull_chart(gmPerson.cPerson(pk_patient))
-			if emr is None:
-				return False
+			emr = gmClinicalRecord.cClinicalRecord(aPKey = pk_patient)
 			epi = emr.add_episode(episode_name = 'administrative', is_open = False)
 			soap_cat = gmTools.bool2subst (
 				(self.__focussed_msg['category'] == u'clinical'),
